/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
 
 
 // https://github.com/paypal/PayPal-Cordova-Plugin
 
var app = {
	// Application Constructor
	initialize: function() {
		this.bindEvents();
	},
	// Bind Event Listeners
	//
	// Bind any events that are required on startup. Common events are:
	// 'load', 'deviceready', 'offline', and 'online'.
	bindEvents: function() {
		document.addEventListener('deviceready', this.onDeviceReady, false);
	},
	// deviceready Event Handler
	//
	// The scope of 'this' is the event. In order to call the 'receivedEvent'
	// function, we must explicitly call 'app.receivedEvent(...);'
	onDeviceReady: function() {
		app.receivedEvent('deviceready');

		init();

	},
	// Update DOM on a Received Event
	receivedEvent: function(id) {
		var parentElement = document.getElementById(id);
		var listeningElement = parentElement.querySelector('.listening');
		var receivedElement = parentElement.querySelector('.received');

		listeningElement.setAttribute('style', 'display:none;');
		receivedElement.setAttribute('style', 'display:block;');

		console.log('Received Event: ' + id);
	}
};

// edition
var applicationDate = "2017-05-18a";
var FREE = true;
var FREEPERCENTILE = .25;	// only REALLY COMMON WORDS

// constants and common stuff
var letters = ['a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z'];
var letterbgcolors = ['#ffffff', '#a0a0a0', '#a0a0ff', '#ffa0a0'];
var lettercolors = ['#000000', '#000000', '#000000', '#000000'];
var allowBlueWords;

// game play
var ptype = ['',''];			// player type (set at startGame())
var pskill = ['',''];			// computer skill levels (1-6) (set at startGame())
var pword = ['',''];			// player word (set at startGame())
var pguess = [[],[]];			// player guessed (set at each round)
var pscore = [[],[]];			// player score (set at each round)
var pwords = [[],[]];			// player private word list
var p = 0;						// current player
var pletters = [[],[]];			// color status (0...) of each of the letters for display



// cause a section to show and hide all others
function show(id) {
	var s = document.querySelectorAll(".oneatatime");
	for(var i=0; i<s.length; i++) {
		if (s[i].id==id) {
			s[i].style.display = "inherit";
		} else {
			s[i].style.display = "none";
		}
	}
}

function init() {
	// load in settings
	allowBlueWords = localStorage.getItem("allowbluewords");

	// build a few things
	buildPercentileAttributeInWords();

	// setup the screens and stuff for the free version or paid version
	var a = document.querySelectorAll(".freedisplay");
	for (var i =0; i < a.length; i++) {
		a[i].style.display = (FREE ? "inherit" : "none");
	}
	var a = document.querySelectorAll(".paiddisplay");
	for (var i =0; i < a.length; i++) {
		a[i].style.display = (!FREE ? "inherit" : "none");
	}

	// initialize the welcome screen
	document.querySelector("#ptypeh0").checked = true;
	showhidepwordarea(0);
	document.querySelector("#ptypec1").checked = true;
	showhidepwordarea(1);



	// kick the play off
	show('welcome');
}

function buildPercentileAttributeInWords() {
	// percentile difficulty of word
	// 0 is really common - 1 is rare
	var f = [];
	for (var w in words) {
		f.push(words[w].f);
	}
	f.sort(function(a, b){return b-a});    // sort the numbers in desc order
	//console.log(f);
	for (var w in words) {
		var pos = f.indexOf(words[w].f);
		words[w].percentile = pos/f.length;
		//console.log(w + " " + words[w].f + " " + pos + " " + words[w].percentile);
	}
	//console.log(words);
}

function getPlayerVocab(player) {
	// calculate the strength of the player's vocabulary (based on the words used)
	// pass player number (so you can calculate your opponents)
	var v = .25;
	for(var i=0; i<pguess[player].length; i++) {
		v += words[pguess[player][i]].percentile;
	}
	v = v / (pguess[player].length+1);
	console.log("Player " + player + " vocab " + v);
	return v;
}  

function showSettings() {
	var e = document.querySelector("#settingsallowbluewords");
	e.checked = !(!localStorage.getItem("allowbluewords"));
	show('settings');
}

function updateSettings() {
	var e = document.querySelector("#settingsallowbluewords");
	allowBlueWords = e.checked;
	if (allowBlueWords) {
		localStorage.setItem('allowbluewords',1);
	} else {
		localStorage.removeItem('allowbluewords');
	}
	alert("Settings Changed.");
	show('welcome');
}

function showAbout() {
	document.querySelector("#applicationDate").innerHTML = applicationDate;
	if (typeof AppVersion != 'undefined') {
		document.querySelector("#applicationVersion").innerHTML = AppVersion.version;
		document.querySelector("#applicationBuild").innerHTML = AppVersion.build;
	}
	show('about');
}

function openURL(url) {
	var ref = cordova.InAppBrowser.open(url, '_blank', 'location=yes');
}

function openRENEJMURL() {
	var url = "http://www.renejm.com";
	openURL(url);
}

function calculateFrequency(arrayOfWords) {
	// build letter frequency table for an array of words
	// object with letter being the key to count
	f = {};
	for (var n=0; n<letters.length; n++) {
		f[letters[n]] = 0;
	}
	for (var n=0; n<arrayOfWords.length; n++) {
		var w = arrayOfWords[n];
		for (var i=0; i<4; i++) {
			f[w.charAt(i)] ++;
		}
	}
	return f;
}

function showhidepwordarea(n) {
	var w = document.querySelector("#pwordarea"+n);
	var s = document.querySelector("#pskillarea"+n);
	if (document.querySelector("#ptypeh"+n).checked) {
		w.style.display = 'inherit';
		s.style.display = 'none';
	} else {
		w.style.display = 'none';
		s.style.display = 'inherit';
	}
}

function getComputerSecret(n) {
	// need a better algorithm than this...
	// pwords[n] contains all words at this point
	return pwords[n][Math.floor(Math.random()*pwords[n].length)];
}

function getComputerGuessRandom() {
	// simply return a random word from the list of words that the computer
	// believes to contain the word
	return pwords[p][Math.floor(Math.random()*pwords[p].length)];
}

function getComputerGuessMaxFreq() {
	// get the computer guess that has the highest frequency of letters
	// remaining in the computer's list of words (that contains the
	// other player's word).  if there are multiple words with the same
	// high - then return a random one
	//
	var freqs = calculateFrequency(pwords[p])
	var bestw = [];
	var bestf = -999999999999;
	//
	for(n=0; n<pwords[p].length;n++) {
		var w = pwords[p][n];
		f = freqs[w.charAt(0)] + freqs[w.charAt(1)]
			+ freqs[w.charAt(2)] + freqs[w.charAt(3)];
		if (f>bestf) {
			bestf = f;
			bestw = [];
		}
		if (f==bestf) {
			bestw.push(w);
		}
	}
	return bestw[Math.floor(Math.random()*bestw.length)];
}

function getComputerGuessVocab() {
	// calculate the opponent's vocabulary percentile (of words used so far)
	// and find the word with the closest percentile score
	//
	var vocab = getPlayerVocab(otherp());
	var freqs = calculateFrequency(pwords[p])
	var bestw = '';
	var lowvariance = 1;
	//
	for(n=0; n<pwords[p].length;n++) {
		var w = pwords[p][n];
		var variance =  Math.pow(vocab - words[pwords[p][n]].percentile, 2);
		if (variance < lowvariance) {
			lowvariance = variance;
			bestw = w;
		}
	}
	return bestw;
}

function getComputerGuess() {
	return getComputerGuessVocab();
}


function trimComputerWords(word, score) {
	// pass guessed word an score and remove words that do
	// not have the same score (they are useless)
	//
	// also use stupid score to leave a percentage of incorrect words
	// in the array (to give humans a better chance)
	var stupid;
	if (pskill[p] >= 5) {
		stupid = 0;		//no stupidity
	} else if (pskill[p] == 4) {
		stupid = .20;
	} else if (pskill[p] == 3) {
		stupid = .50;
	} else if (pskill[p] == 2) {
		stupid = .30;
	} else {
		stupid = .10;
	}
	//
	for(var n = pwords[p].length-1; n>=0; n--) {
		if (word==pwords[p][n]) {
			pwords[p].splice(n,1);
		} else {
			if(stupid < Math.random() && compareWords(word,pwords[p][n]) != score) {
				pwords[p].splice(n,1);
			}
		}
	}
	console.log("player " + p + " (" + pskill[p] +") has " + pwords[p].length + " words left.")
}

function arrayOfWords() {
	// create an array (copy) of the words (keys)
	// in FREE version only build a truncated list
	b = [];
	for(var w in words) {
		if (isPlayableWord(w)) b.push(w);
	}
	console.log("word array length " + b.length);
	return b;
}

function isPlayableWord(w) {
	// return boolean if the blue check and free check said this
	// is an Ok word to play
	if (!(w in words)) return false;
	if (!allowBlueWords && "blue" in words[w]) return false;
	if (FREE && words[w].percentile > FREEPERCENTILE) {
		//console.log(w + " " + words[w].percentile + " " + FREEPERCENTILE);
		return false;
	}
	return true;
}
	
function checkUserWord(w) {
	// check to see if w is in the master dictionary
	// display message - return boolean
	if (!isPlayableWord(w)) {
		if (FREE) {
			if ((w in words)) {
				alert("Your word is not available in the 'free' version, but is in the paid version. Select another.");
			} else {
				alert("Your word is not in the 'free' dictionary. Select another.");
			}
		} else {
			alert("Your word is not in the dictionary. Select another.");
		}
		return false;
	}
	return true;
}

function startGame() {
	// handle the start of the game.  Get the human's secret and validate
	// or get the computer's secret.  Initialize the pscore, pguess, and
	// other arrays
	for(var n=0; n<=1; n++) {
		if (document.querySelector("#ptypeh"+n).checked) {
			var element = document.querySelector("#pword"+n);
			ptype[n] = 'h';
			pskill[n] = 0;
			pword[n] = element.value.toLowerCase();
			pwords[n] = [];
			if(!checkUserWord(pword[n])) return;
			element.value = "";
		} else {
			ptype[n] = 'c';
			pskill[n] = parseFloat(document.querySelector("#pskill"+n).value);
			pwords[n] = arrayOfWords();
			pword[n] = getComputerSecret(n);
		}
		pguess[n] = [];
		pscore[n] = [];
		for(var m=0; m<letters.length; m++) {
			pletters[n][m] = 0;
		}
	}
	p = 1;
	doPlayer();
}

function doPlayer() {
	// change to the next player and either start a human or computer turn
	p = otherp();
	if (ptype[p]=='h') {
		humanTurn();
	} else {
		computerTurn();
	}
}

function otherp() {
	// without changing player - what is the other player number
	if (p==0) return 1;
	return 0;
}

function compareWords(a,b) {
	//compare two word and return the number of common letters
	var kount = 0;
	for(i=0; i<a.length; i++) {
		for(j=0; j<b.length; j++) {
			if (a.charAt(i) == b.charAt(j)) kount++;
		}
	}
	//alert(a+" "+b+" "+kount)
	return kount;
}

function showGuesses(pageid) {
	// show a player's previous guesses on the UI
	var list = document.querySelector(pageid + " .guesses");
	while (list.hasChildNodes()) {
		list.removeChild(list.lastChild);
	}
	for (var n=0; n<pguess[p].length; n++) {
		var item = document.createElement("li");
		var ls = '';
		for (var m=0;m<4;m++) {
			ls += "<span class='CHR"+pguess[p][n].charAt(m)+"'>"+pguess[p][n].charAt(m)+"</span>";
		}
		item.innerHTML = ls + " - " + pscore[p][n];
		list.appendChild(item);
	}
}

function showLetters(pageid) {
	// create the letter buttons HTML on the UI tht the user can tap on
	// also update the once they go out so that the previous colors
	// are restored
	var para = document.querySelector(pageid + " .letterarea");
	para.innerHTML="";
	for(var n=0; n<letters.length; n++) {
		if (pletters[p][n]>=lettercolors.length) pletters[p][n] = 0;
		var l = "<span onclick='pletters["+p+"]["+n+"]++;updateLetters(\""+pageid+"\");' class='CHR"+letters[n]+" letters'>"+letters[n]+"</span>";
		para.innerHTML+=l
	}
	updateLetters(pageid);
}

function wordSearch() {
	// show a player's previous guesses
	var list = document.querySelector("#wordsearch .words");
	var searchfor = document.querySelector("#wordsearch .word").value.toLowerCase();
	// clear the old word list
	while (list.hasChildNodes()) {
		list.removeChild(list.lastChild);
	}
	// find the best words
	var foundwords=[];
	var score=0;
	var maxscore=0;
	for (var w in words) {
		if (isPlayableWord(w)) {
			score = compareWords(w,searchfor);
			if (score > maxscore) {
				maxscore = score;
				foundwords = [];
			}
			if (score==maxscore) {
				foundwords.push(w);
			}
		}
	}
	// display words (max 10 or 30)
	for (var n=0; n<foundwords.length&&n<(FREE?10:30); n++) {
		var item = document.createElement("li");
		item.innerHTML = foundwords[n];
		list.appendChild(item);
	}
}


function updateLetters() {
	// update letters in guess history and letter list when one is changed
	// this is used to handle the tap on a letter in the UI
	for(var n=0; n<letters.length; n++) {
		if (pletters[p][n]>=lettercolors.length) pletters[p][n] = 0;
		var a = document.querySelectorAll(".CHR" + letters[n]);
		for (var i=0; i<a.length; i++) {
			a[i].style.backgroundColor = letterbgcolors[pletters[p][n]]; 
			a[i].style.color = lettercolors[pletters[p][n]]; 
		}
	}
}

function humanTurn() {
	// start the human turn by asking then for a word
	document.querySelector("#humanentry .playerno").innerHTML = p+1;
	document.querySelector("#humanentry .word").value = "";
	showGuesses("#humanentry");
	showLetters("#humanentry");
	show("humanentry");
}

function humanTurnEntry() {
	var word = document.querySelector("#humanentry .word").value.toLowerCase();
	if(!checkUserWord(word)) return;
	if (pguess[p].indexOf(word)!=-1) {
		alert("Your have used this word already - select another.");
		return;
	}
	var score = compareWords(word, pword[otherp()]);
	pguess[p].push(word);
	pscore[p].push(score);
	if (word==pword[otherp()]) {
		// game over
		saveHistory();
		document.querySelector("#humanwin .playerno").innerHTML = p+1;
		document.querySelector("#humanwin .word").innerHTML = word;
		document.querySelector("#humanwin .secret").innerHTML = pword[p];
		showGuesses("#humanwin");
		showLetters("#humanwin");
		show("humanwin");
	} else {
		document.querySelector("#humanscore .playerno").innerHTML = p+1;
		document.querySelector("#humanscore .word").innerHTML = word;
		document.querySelector("#humanscore .score").innerHTML = score;
		showGuesses("#humanscore");
		showLetters("#humanscore");
		show("humanscore");
	}
}

function computerTurn() {
	// get the computer word
	// calculate, process, and display
	var word = getComputerGuess();
	var score = compareWords(word, pword[otherp()]);
	trimComputerWords(word, score);
	//
	// add to history arrays
	pguess[p].push(word);
	pscore[p].push(score);
	//
	// build letters (colors) to show the human how the computer is progressing
	var f = calculateFrequency(pwords[p]);
	for(var n=0; n<letters.length; n++) {
		pletters[p][n] = parseInt(f[letters[n]] / pwords[p].length * letterbgcolors.length);
		if (pletters[p][n]==letterbgcolors.length) pletters[p][n]--;
	}
	//
	if (word==pword[otherp()]) {
		// game over
		saveHistory();
		document.querySelector("#computerwin .playerno").innerHTML = p+1;
		document.querySelector("#computerwin .word").innerHTML = word;
		document.querySelector("#computerwin .secret").innerHTML = pword[p];
		showGuesses("#computerwin");
		show("computerwin");
	} else {
		document.querySelector("#computerscore .playerno").innerHTML = p+1;
		document.querySelector("#computerscore .word").innerHTML = word;
		document.querySelector("#computerscore .score").innerHTML = score;
		showGuesses("#computerscore");
		showLetters("#computerscore");
		show("computerscore");
	}
}

function getDate() {
	// return date as a string
	var date = new Date();
	return date.getFullYear() + "-" + (date.getMonth()<9?"0":"") + (date.getMonth()+1)
		+ "-" +  (date.getDate()<10?"0":"") + (date.getDate());
}

function saveHistory() {
	var history= localStorage.getItem("w4History");
	if(!history) {
		history= [];
	} else {
		history = JSON.parse(history);
	}
	entry = {}
	entry["date"] = getDate();
	for (var n=0; n<=1; n++) {
		entry["ptype"+n] = ptype[n];
		if (pskill[n]) entry["ptype"+n] += " (" + pskill[n] + ")";
		entry["pword"+n] = pword[n];
	}
	entry["winner"] = p;
	entry["guesses"] = pguess[p].length;
	history.push(entry);
	history = JSON.stringify(history);
	localStorage.setItem("w4History",history);
}

function showHistory() {
	// fill the history section and show
	var history= localStorage.getItem("w4History");
	if(!history) {
		history= [];
	} else {
		history = JSON.parse(history);
	}
	var t = document.querySelector("#history .table");
	x = "<tr><th>Date</th><th>P1</th><th>Word</th><th>P2</th><th>Word</th><th>Win</th><th>#</th></tr>";
	for (var n=history.length-1; n>=0; n--) {
		x += "<tr><td>"+history[n].date+"</td>";
		x += "<td>"+history[n].ptype0+"</td>";
		x += "<td>"+history[n].pword0+"</td>";
		x += "<td>"+history[n].ptype1+"</td>";
		x += "<td>"+history[n].pword1+"</td>";
		x += "<td>"+(history[n].winner+1)+"</td>";
		x += "<td>"+history[n].guesses+"</td></tr>";
	}
	t.innerHTML = x;
	show("history");
}

